@@ -24,6 +24,8 @@ from requests.packages.urllib3.util import Retry
 import logging
 import websocket
 import json
+import time
+from pydantic.error_wrappers import ValidationError
 
 # endregion
 
@@ -34,24 +36,52 @@ logger = logging.getLogger(__name__)
 # endregion
 
 # region LocalImports
+from escriptorium_connector.utils import TimeoutWebsocket
+from escriptorium_connector.connector_errors import (
+    EscriptoriumConnectorHttpError,
+    EscriptoriumConnectorDtoSyntaxError,
+    EscriptoriumConnectorDtoTypeError,
+    EscriptoriumConnectorDtoValidationError,
+)
 from escriptorium_connector.dtos import (
     GetProjects,
     GetProject,
     PostProject,
     PutProject,
-    GetDocument,
     GetDocuments,
+    GetDocument,
     PostDocument,
     PutDocument,
+    GetParts,
+    GetPart,
+    PostPart,
+    PutPart,
+    GetLines,
+    GetLine,
+    PostLine,
+    PutLine,
+    GetLineTypes,
+    GetLineType,
+    PostLineType,
     GetAnnotationTaxonomy,
     GetAnnotationTaxonomies,
     PostAnnotationTaxonomy,
     PagenatedResponse,
     GetUser,
+    GetRegions,
+    GetRegion,
+    PostRegion,
+    GetRegionTypes,
     GetRegionType,
+    PostRegionType,
     GetComponent,
     GetComponents,
     PostComponent,
+    GetAbbreviatedTranscription,
+    PostAbbreviatedTranscription,
+    GetTranscription,
+    GetTranscriptions,
+    PostTranscription,
 )
 
 # endregion
@@ -218,18 +248,18 @@ class EscriptoriumConnector:
     # endregion
 
     # region Websockets (not used)
-    def __on_message(self, ws, message):
+    def __on_message(self, ws, message):  # pragma: no cover
         logging.debug(message)
 
-    def __on_error(self, ws, error):
+    def __on_error(self, ws, error):  # pragma: no cover
         logging.debug(error)
 
-    def __on_close(self, ws, close_status_code, close_msg):
+    def __on_close(self, ws, close_status_code, close_msg):  # pragma: no cover
         logging.debug("### websocket closed ###")
         logging.debug(close_status_code)
         logging.debug(close_msg)
 
-    def __on_open(self, ws):
+    def __on_open(self, ws):  # pragma: no cover
         logging.debug("### websocket opened ###")
 
     # endregion
@@ -239,13 +269,17 @@ class EscriptoriumConnector:
         return self.http.get(url)
 
     def __post_url(
-        self, url: str, payload: dict, files: object = None
+        self, url: str, payload: dict, files: object = None, as_form_data: bool = False
     ) -> requests.Response:
         prepared_payload = json.loads(json.dumps(payload, cls=EnhancedJSONEncoder))
         return (
             self.http.post(url, data=prepared_payload, files=files)
             if files is not None
-            else self.http.post(url, json=prepared_payload)
+            else (
+                self.http.post(url, data=prepared_payload)
+                if as_form_data
+                else self.http.post(url, json=prepared_payload)
+            )
         )
 
     def __put_url(
@@ -261,35 +295,47 @@ class EscriptoriumConnector:
     def __delete_url(self, url: str) -> requests.Response:
         return self.http.delete(url)
 
-    def __get_url_serialized(self, url: str, cls: Type[T]) -> T:
-        r = self.http.get(url)
-        r_json = r.json()
-        obj = cls(**r_json)
+    def __serialize_response(
+        self, response: requests.Response, url: str, return_cls: Type[T]
+    ) -> T:
+        r_json = response.json()
+        try:
+            obj = return_cls(**r_json)
+        except SyntaxError as e:
+            raise EscriptoriumConnectorDtoSyntaxError(
+                e, response.status_code, url, response.text
+            )
+        except TypeError as e:
+            raise EscriptoriumConnectorDtoTypeError(
+                e, response.status_code, url, response.text
+            )
+        except ValidationError as e:
+            raise EscriptoriumConnectorDtoValidationError(
+                e, response.status_code, url, response.text
+            )
         return obj
 
+    def __get_url_serialized(self, url: str, return_cls: Type[T]) -> T:
+        r = self.http.get(url)
+        return self.__serialize_response(r, url, return_cls)
+
     def __post_url_serialized(
         self, url: str, payload: dict, return_cls: Type[T], files: object = None
     ) -> T:
         r = self.__post_url(url, payload, files)
-        r_json = r.json()
-        obj = return_cls(**r_json)
-        return obj
+        return self.__serialize_response(r, url, return_cls)
 
     def __put_url_serialized(
         self, url: str, payload: dict, return_cls: Type[T], files: object = None
     ) -> T:
         r = self.__put_url(url, payload, files)
-        r_json = r.json()
-        obj = return_cls(**r_json)
-        return obj
+        return self.__serialize_response(r, url, return_cls)
 
-    def __get_paginated_response(self, url: str, cls: Type[P]) -> P:
-        r = self.__get_url(url)
-        all_docs = cls(**json.loads(r.text))
+    def __get_paginated_response(self, url: str, return_cls: Type[P]) -> P:
+        all_docs = self.__get_url_serialized(url, return_cls)
         info = all_docs
         while info.next is not None:
-            r = self.__get_url(info.next)
-            info = cls(**json.loads(r.text))
+            info = self.__get_url_serialized(info.next, return_cls)
             all_docs.results = all_docs.results + info.results
 
         return all_docs
@@ -326,6 +372,14 @@ class EscriptoriumConnector:
             f"{self.api_url}projects/", asdict(project_data), GetProject
         )
 
+    # def create_project(self, project_name: str) -> Union[GetProject, None]:
+    #     new_project_data = PostProject(
+    #         name=project_name, csrfmiddlewaretoken=self.csrfmiddlewaretoken
+    #     )
+    #     r = self.http.post(f"{self.base_url}projects/", data=asdict(new_project_data), headers=self.http.headers)
+    #     project_pk = self.get_project_pk_by_name(project_name)
+    #     return self.get_project(project_pk) if project_pk is not None else None
+
     def update_project(self, project_data: PutProject) -> GetProject:
         return self.__put_url_serialized(
             f"{self.api_url}projects/", asdict(project_data), GetProject
@@ -355,31 +409,118 @@ class EscriptoriumConnector:
             f"{self.api_url}documents/", asdict(doc_data), GetDocument
         )
 
-    def update_document(self, doc_data: PutDocument) -> GetDocument:
+    def update_document(self, pk: int, doc_data: PutDocument) -> GetDocument:
         return self.__put_url_serialized(
-            f"{self.api_url}documents/", asdict(doc_data), GetDocument
+            f"{self.api_url}documents/{pk}/", asdict(doc_data), GetDocument
         )
 
     def delete_document(self, pk: int):
-        return self.__delete_url(f"{self.api_url}documents/{pk}")
+        return self.__delete_url(f"{self.api_url}documents/{pk}/")
 
     # endregion
 
     # region Part API
-    def get_document_parts(self, doc_pk: int):
-        return self.get_document_images(doc_pk)
+    def get_document_parts(self, doc_pk: int) -> GetParts:
+        return self.__get_paginated_response(
+            f"{self.api_url}documents/{doc_pk}/parts/", GetParts
+        )
 
-    def get_document_part(self, doc_pk: int, part_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/")
-        return r.json()
+    def get_document_part(self, doc_pk: int, part_pk: int) -> GetPart:
+        return self.__get_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/", GetPart
+        )
 
-    def delete_document_parts(self, document_pk: int, start: int, end: int):
-        parts = self.get_document_images(document_pk)
-        for part in parts[start:end]:
-            r = self.__delete_url(
-                f'{self.api_url}documents/{document_pk}/parts/{part["pk"]}/'
+    def get_document_part_image(self, doc_pk: int, part_pk: int) -> bytes:
+        part = self.get_document_part(doc_pk, part_pk)
+        return self.get_image(part.image.uri)
+
+    def get_document_part_thumbnail(
+        self, doc_pk: int, part_pk: int, large: bool = False
+    ) -> bytes:
+        # The thumbnails are not immediately generated,
+        # retry a couple times before giving up
+        num_tries = 5
+        current_try = 0
+        part = self.get_document_part(doc_pk, part_pk)
+        needed_size_exists = (
+            part.image.thumbnails.large is not None
+            if large
+            else part.image.thumbnails.card is not None
+        )
+
+        while not needed_size_exists and current_try < num_tries:
+            time.sleep(4)
+            part = self.get_document_part(doc_pk, part_pk)
+            needed_size_exists = (
+                part.image.thumbnails.large is not None
+                if large
+                else part.image.thumbnails.card is not None
             )
 
+        image_url = part.image.thumbnails.large if large else part.image.thumbnails.card
+        if image_url is None:
+            raise Exception(
+                f"""Could not find a {"large" if large else "card"} thumbnail for doc {doc_pk}, part {part_pk}"""
+            )
+
+        return self.get_image(image_url)
+
+    def create_document_part(
+        self,
+        document_pk: int,
+        image_data_info: PostPart,
+        filename: str,
+        image_data: bytes,
+    ) -> GetPart:
+        return self.__post_url_serialized(
+            f"{self.api_url}documents/{document_pk}/parts/",
+            asdict(image_data_info),
+            GetPart,
+            {"image": (filename, image_data)},
+        )
+
+    def update_document_part(
+        self,
+        doc_pk: int,
+        part_pk: int,
+        image_data_info: PutPart,
+    ) -> GetPart:
+        return self.__put_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/",
+            asdict(image_data_info),
+            GetPart,
+        )
+
+    def delete_document_part(self, doc_pk: int, part_pk: int):
+        self.__delete_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/")
+
+    def delete_document_parts_by_index(self, doc_pk: int, start: int, end: int):
+        """Deletes N parts from the documents, starting with and including the
+        item at the `start` index and ending with but *not* including the item at
+        the `end` index.
+
+        Args:
+            doc_pk (int): PK of the document from which to remove the parts
+            start (int): Index of the first part to be removed according to the
+            order in which the document parts are returned (check the part `order`
+            attribute to be sure)
+            end (int): The index at which to stop deleting document partsaccording
+            to the order in which the document parts are returned (check the part
+            `order` attribute to be sure). The item at this index (if it exists)
+            will not be deleted.
+        """
+
+        if start > end:
+            return
+
+        parts = (self.get_document_parts(doc_pk)).results
+
+        if start > len(parts) < end:
+            return
+
+        for part in parts[start:end]:
+            self.delete_document_part(doc_pk, part.pk)
+
     # endregion
 
     # region Up/Download API
@@ -465,11 +606,12 @@ class EscriptoriumConnector:
             raise Exception("Must use websockets to download ALTO exports")
 
         download_link = None
-        ws = websocket.WebSocket()
+        ws = TimeoutWebsocket()
         ws.connect(
-            f"{self.base_url.replace('http', 'ws')}ws/notif/", cookie=self.cookie
+            f"{self.base_url.replace('http', 'ws')}ws/notif/",
+            cookie=self.cookie,
         )
-        self.__post_url(
+        r = self.__post_url(
             f"{self.api_url}documents/{document_pk}/export/",
             {
                 "task": "export",
@@ -482,9 +624,10 @@ class EscriptoriumConnector:
                 "document": document_pk,
                 "parts": part_pk,
             },
+            as_form_data=True,
         )
 
-        message = ws.recv()
+        message = ws.recv(30)
         ws.close()
         logging.debug(message)
         msg = json.loads(message)
@@ -538,227 +681,207 @@ class EscriptoriumConnector:
 
     # endregion
 
-    # region Image Up/Download
+    # region Image Download
 
-    def get_document_images(self, document_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{document_pk}/parts/")
-        image_info = r.json()
-        image_names = image_info["results"]
-        while image_info["next"] is not None:
-            r = self.__get_url(image_info["next"])
-            image_info = r.json()
-            image_names = image_names + image_info["results"]
-
-        return image_names
-
-    def get_image(self, img_url: str):
-        r = self.__get_url(f"{self.base_url}{img_url}")
+    def get_image(self, img_url: str) -> bytes:
+        sanitized_img_url = img_url.lstrip("/")
+        r = self.__get_url(f"{self.base_url}{sanitized_img_url}")
         return r.content
 
-    def create_image(
-        self, document_pk: int, image_data_info: object, image_data: bytes
-    ):
-        return self.__post_url(
-            f"{self.api_url}documents/{document_pk}/parts/",
-            image_data_info.__dict__,
-            {"image": (image_data_info["filename"], image_data)},
-        )
-
     # endregion
 
     # endregion
 
     # region Line API
 
-    def get_document_part_line(self, doc_pk: int, part_pk: int, line_pk: int):
-        r = self.__get_url(
-            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}/"
+    def get_document_part_line(
+        self, doc_pk: int, part_pk: int, line_pk: int
+    ) -> GetLine:
+        return self.__get_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}/",
+            GetLine,
         )
-        return r.json()
 
-    def get_document_part_lines(self, doc_pk: int, part_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/")
-        line_info = r.json()
-        lines = line_info["results"]
-        while line_info["next"] is not None:
-            r = self.__get_url(line_info["next"])
-            line_info = r.json()
-            lines = lines + line_info["results"]
+    def get_document_part_lines(self, doc_pk: int, part_pk: int) -> GetLines:
+        return self.__get_paginated_response(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/", GetLines
+        )
 
-        return lines
-
-    def create_document_part_line(self, doc_pk: int, part_pk: int, new_line: object):
-        r = self.__post_url(
+    def create_document_part_line(
+        self, doc_pk: int, part_pk: int, new_line: PostLine
+    ) -> GetLine:
+        return self.__post_url_serialized(
             f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/",
-            new_line.__dict__,
+            asdict(new_line),
+            GetLine,
         )
-        return r
 
-    def delete_document_part_line(self, doc_pk: int, part_pk: int, line_pk: int):
-        r = self.__delete_url(
+    def update_document_part_line(
+        self, doc_pk: int, part_pk: int, line_pk: int, new_line: PutLine
+    ) -> GetLine:
+        return self.__put_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}/",
+            asdict(new_line),
+            GetLine,
+        )
+
+    def delete_document_part_line(
+        self, doc_pk: int, part_pk: int, line_pk: int
+    ) -> requests.Response:
+        return self.__delete_url(
             f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}"
         )
-        return r
 
     # endregion
 
     # region Line Type API
 
-    def get_line_types(self):
-        r = self.__get_url(f"{self.api_url}types/line/")
-        line_type_info = r.json()
-        line_types = line_type_info["results"]
-        while line_type_info["next"] is not None:
-            r = self.__get_url(line_type_info["next"])
-            line_type_info = r.json()
-            line_types = line_types + line_type_info["results"]
+    def get_document_line_types(self, doc_pk: int) -> List[GetLineType]:
+        doc_data = self.get_document(doc_pk)
+        return [x for x in doc_data.valid_line_types]
 
-        return line_types
+    def get_line_types(self) -> GetLineTypes:
+        return self.__get_paginated_response(f"{self.api_url}types/line/", GetLineTypes)
 
-    def create_line_type(self, line_type: object):
-        r = self.__post_url(f"{self.api_url}types/line/", line_type.__dict__)
-        return r
+    def get_line_type(self, line_type_pk: int) -> GetLineType:
+        return self.__get_url_serialized(
+            f"{self.api_url}types/line/{line_type_pk}/", GetLineType
+        )
+
+    def create_line_type(self, line_type: PostLineType) -> GetLineType:
+        return self.__post_url_serialized(
+            f"{self.api_url}types/line/", asdict(line_type), GetLineType
+        )
+
+    def update_line_type(
+        self, line_type_pk: int, line_type: PostLineType
+    ) -> GetLineType:
+        return self.__put_url_serialized(
+            f"{self.api_url}types/line/{line_type_pk}/", asdict(line_type), GetLineType
+        )
+
+    def delete_line_type(self, line_type_pk: int) -> requests.Response:
+        return self.__delete_url(f"{self.api_url}types/line/{line_type_pk}/")
 
     # endregion
 
     # region Region API
 
-    def get_document_part_region(self, doc_pk: int, part_pk: int, region_pk: int):
-        regions = self.get_document_part_regions(doc_pk, part_pk)
-        region = [x for x in regions if x["pk"] == region_pk]
+    def get_document_part_region(
+        self, doc_pk: int, part_pk: int, region_pk: int
+    ) -> Union[GetRegion, None]:
+        regions = (self.get_document_part_regions(doc_pk, part_pk)).results
+        region = [x for x in regions if x.pk == region_pk]
         return region[0] if region else None
 
-    def get_document_part_regions(self, doc_pk: int, part_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}")
-        part = r.json()
-        return part["regions"]
+    def get_document_part_regions(self, doc_pk: int, part_pk: int) -> GetRegions:
+        return self.__get_paginated_response(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}", GetRegions
+        )
 
-    def create_document_part_region(self, doc_pk: int, part_pk: int, region: object):
-        if not isinstance(region["box"], str):
-            region["box"] = json.dumps(region["box"])
-
-        r = self.__post_url(
+    def create_document_part_region(
+        self, doc_pk: int, part_pk: int, region: PostRegion
+    ) -> GetRegion:
+        return self.__post_url_serialized(
             f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/blocks/",
-            region.__dict__,
+            asdict(region),
+            GetRegion,
         )
-        return r
 
     # endregion
 
     # region Region Type API
 
-    def get_region_types(self):
-        r = self.__get_url(f"{self.api_url}types/block/")
-        block_type_info = r.json()
-        block_types = block_type_info["results"]
-        while block_type_info["next"] is not None:
-            r = self.__get_url(block_type_info["next"])
-            block_type_info = r.json()
-            block_types = block_types + block_type_info["results"]
-
-        return block_types
+    def get_region_types(self) -> GetRegionTypes:
+        return self.__get_paginated_response(
+            f"{self.api_url}types/block/", GetRegionTypes
+        )
 
     def get_document_region_types(self, document_pk: int) -> List[GetRegionType]:
         doc_data = self.get_document(document_pk)
         return [x for x in doc_data.valid_block_types]
 
-    def create_region_type(self, region_type: object):
-        r = self.__post_url(f"{self.api_url}types/block/", region_type.__dict__)
-        return r
+    def create_region_type(self, region_type: PostRegionType) -> GetRegionType:
+        return self.__post_url_serialized(
+            f"{self.api_url}types/block/", asdict(region_type), GetRegionType
+        )
 
     # endregion
 
     # region Transcription API
     def get_document_part_line_transcription(
         self, doc_pk: int, part_pk: int, line_pk: int, line_transcription_pk: int
-    ):
+    ) -> Union[GetTranscription, None]:
         transcriptions = self.get_document_part_line_transcriptions(
             doc_pk, part_pk, line_pk
         )
-        transcription = [x for x in transcriptions if x["pk"] == line_transcription_pk]
+        transcription = [x for x in transcriptions if x.pk == line_transcription_pk]
         return transcription[0] if transcription else None
 
     def get_document_part_line_transcription_by_transcription(
         self, doc_pk: int, part_pk: int, line_pk: int, transcription_pk: int
-    ):
+    ) -> Union[GetTranscription, None]:
         transcriptions = self.get_document_part_line_transcriptions(
             doc_pk, part_pk, line_pk
         )
         transcription = [
-            x for x in transcriptions if x["transcription"] == transcription_pk
+            x for x in transcriptions if x.transcription == transcription_pk
         ]
         return transcription[0] if transcription else None
 
     def get_document_part_line_transcriptions(
         self, doc_pk: int, part_pk: int, line_pk: int
-    ):
+    ) -> List[GetTranscription]:
         line = self.get_document_part_line(doc_pk, part_pk, line_pk)
-        return line["transcriptions"]
+        return line.transcriptions if line.transcriptions is not None else []
 
-    def get_document_transcription(self, doc_pk: int, transcription_pk: int):
-        r = self.__get_url(
-            f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}/"
+    def get_document_transcription(
+        self, doc_pk: int, transcription_pk: int
+    ) -> GetAbbreviatedTranscription:
+        return self.__get_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}/",
+            GetAbbreviatedTranscription,
         )
-        return r.json()
 
-    def create_document_transcription(self, doc_pk: int, transcription_name: str):
-        r = self.__post_url(
+    def create_document_transcription(
+        self, doc_pk: int, transcription_name: PostAbbreviatedTranscription
+    ) -> GetAbbreviatedTranscription:
+        return self.__post_url_serialized(
             f"{self.api_url}documents/{doc_pk}/transcriptions/",
-            {"name": transcription_name},
+            asdict(transcription_name),
+            GetAbbreviatedTranscription,
         )
-        return r.json()
 
-    def delete_document_transcription(self, doc_pk: int, transcription_pk: int):
-        r = self.__delete_url(
+    def delete_document_transcription(
+        self, doc_pk: int, transcription_pk: int
+    ) -> requests.Response:
+        return self.__delete_url(
             f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}/"
         )
-        return r.json()
 
-    def get_document_transcriptions(self, doc_pk: int):
+    def get_document_transcriptions(
+        self, doc_pk: int
+    ) -> List[GetAbbreviatedTranscription]:
         r = self.__get_url(f"{self.api_url}documents/{doc_pk}/transcriptions/")
-        return r.json()
-
-    def create_document_line_transcription(
-        self,
-        doc_pk: int,
-        parts_pk: int,
-        line_pk: int,
-        transcription_pk: int,
-        transcription_content: str,
-        graphs: Union[Any, None],
-    ):
-        payload = {
-            "line": line_pk,
-            "transcription": transcription_pk,
-            "content": transcription_content,
-        }
-        if graphs is not None:
-            payload["graphs"] = graphs
-        r = self.__post_url(
-            f"{self.api_url}documents/{doc_pk}/parts/{parts_pk}/transcriptions/",
-            payload,
-        )
-        return r.json()
-
-    def get_document_part_transcriptions(self, doc_pk: int, part_pk: int):
-        get_url = f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/transcriptions/"
-        r = self.__get_url(get_url)
-        transcriptions_info = r.json()
-        transcriptions = transcriptions_info["results"]
-        while transcriptions_info["next"] is not None:
-            r = self.__get_url(transcriptions_info["next"])
-            transcriptions_info = r.json()
-            transcriptions = transcriptions + transcriptions_info["results"]
-        return transcriptions
+        r_json = r.json()
+        return [GetAbbreviatedTranscription(**x) for x in r_json]
 
     def create_document_part_transcription(
-        self, doc_pk: int, part_pk: int, transcription: object
-    ):
-        r = self.__post_url(
+        self, doc_pk: int, parts_pk: int, transcription: PostTranscription
+    ) -> GetTranscription:
+        return self.__post_url_serialized(
+            f"{self.api_url}documents/{doc_pk}/parts/{parts_pk}/transcriptions/",
+            asdict(transcription),
+            GetTranscription,
+        )
+
+    def get_document_part_transcriptions(
+        self, doc_pk: int, part_pk: int
+    ) -> GetTranscriptions:
+        return self.__get_paginated_response(
             f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/transcriptions/",
-            transcription.__dict__,
+            GetTranscriptions,
         )
-        return r
 
     # endregion
 
@@ -805,7 +928,7 @@ class EscriptoriumConnector:
             f"{self.api_url}documents/{doc_pk}/taxonomies/components/", GetComponents
         )
 
-    def create_document_components(
+    def create_document_component(
         self, doc_pk: int, component: PostComponent
     ) -> GetComponent:
         return self.__post_url_serialized(
@@ -814,7 +937,7 @@ class EscriptoriumConnector:
             GetComponent,
         )
 
-    def update_document_components(
+    def update_document_component(
         self, doc_pk: int, component_pk: int, component: PostComponent
     ) -> GetComponent:
         return self.__post_url_serialized(
@@ -835,32 +958,3 @@ class EscriptoriumConnector:
         return self.__get_paginated_response(f"""{self.api_url}user""", GetUser)
 
     # endregion
-
-
-# region Package Errors
-class EscriptoriumConnectorHttpError(BaseException):
-    def __init__(self, error_message: str, http_error: requests.HTTPError):
-        self.django_error = error_message
-        self.error = http_error
-
-    def __str__(self):
-        return self.django_error
-
-    def __repr__(self):
-        nl = "\n"
-        return f"""{self.django_error}{nl}{repr(self.error)}"""
-
-
-# endregion
-
-if __name__ == "__main__":
-    import os
-    from dotenv import load_dotenv
-
-    source_url = str(os.getenv("ESCRIPTORIUM_URL"))
-    source_api = f"{source_url}api/"
-    username = str(os.getenv("ESCRIPTORIUM_USERNAME"))
-    password = str(os.getenv("ESCRIPTORIUM_PASSWORD"))
-    project = str(os.getenv("ESCRIPTORIUM_PROJECT"))
-    source = EscriptoriumConnector(source_url, source_api, username, password, project)
-    print(source.get_documents())
