@@ -4,6 +4,7 @@ from .project_dtos import (
     PostProject,
     PutProject,
 )
+
 from .document_dtos import (
     PostDocument,
     PutDocument,
@@ -13,9 +14,40 @@ from .document_dtos import (
     LineOffset,
 )
 
-from .line_dtos import GetLineType
-from .region_dtos import GetRegionType
-from .transcription_dtos import GetTranscription
+from .part_dtos import (
+    GetPart,
+    GetParts,
+    PostPart,
+    PutPart,
+)
+
+from .line_dtos import (
+    GetLines,
+    GetLine,
+    PostLine,
+    PutLine,
+    GetLineTypes,
+    GetLineType,
+    PostLineType,
+)
+
+from .region_dtos import (
+    PostRegionType,
+    GetRegionType,
+    GetRegionTypes,
+    PostRegion,
+    GetRegion,
+    GetRegions,
+)
+
+from .transcription_dtos import (
+    GetAbbreviatedTranscription,
+    PostAbbreviatedTranscription,
+    GetTranscriptions,
+    GetTranscription,
+    PostTranscription,
+)
+
 from .annotation_dtos import (
     TextMarkerType,
     GetAnnotationTaxonomy,
@@ -27,5 +59,7 @@ from .annotation_dtos import (
     PostTypology,
     PostComponent,
 )
+
 from .user_dtos import GetUser, GetOnboarding
+
 from .super_dtos import PagenatedResponse
